//-------------------------------------------------------------------
// Dexter is Copyright (C)2000-2006 Paul Burkey
// All rights reserved. Paul Burkey - paul@shoecakegames.com
//-------------------------------------------------------------------
// dexter-net.h
//
// Portable Utility functions for TCP/IP Networking
//
// This is an optional Module for Dexter. If a project doesn't require
// Networking, this module should not be linked.
//
// ******
// This Module is Currently in prototype stages and is not at all
// portable. Non portable sections will be moved out to the new
// platform-net.cpp module as soon as necessary.
// ******
//-------------------------------------------------------------------

#ifndef _DEXTER_NET_H
#define _DEXTER_NET_H

// Classes defined in this file
class DexterNET;
class SocketTable;

#include "dexter-defines.h"

#include "dexter.h"		// standard Dexter include
#include "dexter-os.h"

const uLong DEX_ERR_OK = 0;			// No Error Occured
const uLong DEX_ERR_NOSOCKET = 1;	// Socket Not Exist

const uLong RECV_BUFF = 4096;

#if defined(DEXTER_WINDOWS)

// Winsock Error Defines

// Ask VisualC to link to these Misc libs if/when they're used
#pragma comment(lib, "ws2_32.lib")	// WinSock32

#define WIN32_LEAN_AND_MEAN
#include <winsock2.h>
#include <Ws2tcpip.h>
#ifdef PeekMessage
#undef PeekMessage
#endif

typedef SOCKET DexSocket;

const uLong DEX_ERR_WOULDBLOCK		= WSAEWOULDBLOCK;
const uLong DEX_ERR_INPROGRESS		= WSAEINPROGRESS;
const uLong DEX_ERR_ALREADY			= WSAEALREADY;
const uLong DEX_ERR_NOTSOCK			= WSAENOTSOCK;
const uLong DEX_ERR_DESTADDRREQ		= WSAEDESTADDRREQ;
const uLong DEX_ERR_MSGSIZE			= WSAEMSGSIZE;
const uLong DEX_ERR_PROTOTYPE		= WSAEPROTOTYPE;
const uLong DEX_ERR_NOPROTOOPT		= WSAENOPROTOOPT;
const uLong DEX_ERR_PROTONOSUPPORT	= WSAEPROTONOSUPPORT;
const uLong DEX_ERR_SOCKTNOSUPPORT	= WSAESOCKTNOSUPPORT;
const uLong DEX_ERR_OPNOTSUPP		= WSAEOPNOTSUPP;
const uLong DEX_ERR_PFNOSUPPORT		= WSAEPFNOSUPPORT;
const uLong DEX_ERR_AFNOSUPPORT		= WSAEAFNOSUPPORT;
const uLong DEX_ERR_ADDRINUSE		= WSAEADDRINUSE;
const uLong DEX_ERR_ADDRNOTAVAIL	= WSAEADDRNOTAVAIL;
const uLong DEX_ERR_NETDOWN			= WSAENETDOWN;
const uLong DEX_ERR_NETUNREACH		= WSAENETUNREACH;
const uLong DEX_ERR_NETRESET		= WSAENETRESET;
const uLong DEX_ERR_CONNABORTED		= WSAECONNABORTED;
const uLong DEX_ERR_CONNRESET		= WSAECONNRESET;
const uLong DEX_ERR_NOBUFS			= WSAENOBUFS;
const uLong DEX_ERR_ISCONN			= WSAEISCONN;
const uLong DEX_ERR_NOTCONN			= WSAENOTCONN;
const uLong DEX_ERR_SHUTDOWN		= WSAESHUTDOWN;
const uLong DEX_ERR_TOOMANYREFS		= WSAETOOMANYREFS;
const uLong DEX_ERR_TIMEDOUT		= WSAETIMEDOUT;
const uLong DEX_ERR_CONNREFUSED		= WSAECONNREFUSED;
const uLong DEX_ERR_LOOP			= WSAELOOP;
const uLong DEX_ERR_NAMETOOLONG		= WSAENAMETOOLONG;
const uLong DEX_ERR_HOSTDOWN		= WSAEHOSTDOWN;
const uLong DEX_ERR_HOSTUNREACH		= WSAEHOSTUNREACH;
const uLong DEX_ERR_NOTEMPTY		= WSAENOTEMPTY;
const uLong DEX_ERR_PROCLIM			= WSAEPROCLIM;
const uLong DEX_ERR_USERS			= WSAEUSERS;
const uLong DEX_ERR_DQUOT			= WSAEDQUOT;
const uLong DEX_ERR_STALE			= WSAESTALE;
const uLong DEX_ERR_REMOTE			= WSAEREMOTE;

#define GetSockErr WSAGetLastError()

#else

// Standard BSD Style error defines

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>

#define INVALID_SOCKET	(DexSocket)(~0)
#define SOCKET_ERROR	(-1)
#define SD_BOTH			SHUT_RDWR	// close red and write
#define SOCKADDR_IN		sockaddr_in
typedef struct sockaddr *LPSOCKADDR;

const uLong DEX_ERR_WOULDBLOCK		= EWOULDBLOCK;
const uLong DEX_ERR_INPROGRESS		= EINPROGRESS;
const uLong DEX_ERR_ALREADY			= EALREADY;
const uLong DEX_ERR_NOTSOCK			= ENOTSOCK;
const uLong DEX_ERR_DESTADDRREQ		= EDESTADDRREQ;
const uLong DEX_ERR_MSGSIZE			= EMSGSIZE;
const uLong DEX_ERR_PROTOTYPE		= EPROTOTYPE;
const uLong DEX_ERR_NOPROTOOPT		= ENOPROTOOPT;
const uLong DEX_ERR_PROTONOSUPPORT	= EPROTONOSUPPORT;
const uLong DEX_ERR_SOCKTNOSUPPORT	= ESOCKTNOSUPPORT;
const uLong DEX_ERR_OPNOTSUPP		= EOPNOTSUPP;
const uLong DEX_ERR_PFNOSUPPORT		= EPFNOSUPPORT;
const uLong DEX_ERR_AFNOSUPPORT		= EAFNOSUPPORT;
const uLong DEX_ERR_ADDRINUSE		= EADDRINUSE;
const uLong DEX_ERR_ADDRNOTAVAIL	= EADDRNOTAVAIL;
const uLong DEX_ERR_NETDOWN			= ENETDOWN;
const uLong DEX_ERR_NETUNREACH		= ENETUNREACH;
const uLong DEX_ERR_NETRESET		= ENETRESET;
const uLong DEX_ERR_CONNABORTED		= ECONNABORTED;
const uLong DEX_ERR_CONNRESET		= ECONNRESET;
const uLong DEX_ERR_NOBUFS			= ENOBUFS;
const uLong DEX_ERR_ISCONN			= EISCONN;
const uLong DEX_ERR_NOTCONN			= ENOTCONN;
const uLong DEX_ERR_SHUTDOWN		= ESHUTDOWN;
const uLong DEX_ERR_TOOMANYREFS		= ETOOMANYREFS;
const uLong DEX_ERR_TIMEDOUT		= ETIMEDOUT;
const uLong DEX_ERR_CONNREFUSED		= ECONNREFUSED;
const uLong DEX_ERR_LOOP			= ELOOP;
const uLong DEX_ERR_NAMETOOLONG		= ENAMETOOLONG;
const uLong DEX_ERR_HOSTDOWN		= EHOSTDOWN;
const uLong DEX_ERR_HOSTUNREACH		= EHOSTUNREACH;
const uLong DEX_ERR_NOTEMPTY		= ENOTEMPTY;
const uLong DEX_ERR_PROCLIM			= EPROCLIM;
const uLong DEX_ERR_USERS			= EUSERS;
const uLong DEX_ERR_DQUOT			= EDQUOT;
const uLong DEX_ERR_STALE			= ESTALE;
const uLong DEX_ERR_REMOTE			= EREMOTE;

#define GetSockErr errno

#endif

#if defined(DEXTER_WINDOWS)
typedef SOCKET DexSocket;
#else
typedef int DexSocket;
#endif

//--------------------------------------------------------------------
// SocketTable Class
//
// Dexters Socket Open Tracking system keeps track of all active Sockets
// When Dexter is finaly shut down, the list of opened sockets are
// checked. Any Sockets that have not been closed are closed and a
// Debug report is made.
//
// Currently Dexter has a fixed number of open sockets. This will
// change to a more flexable linked list in future.
//--------------------------------------------------------------------
class SocketTable {
	public:
	SocketTable() {
		handle = NULL;
		type = 0;
		open = false;
		buff_used = 0;
	}
	bool open;
	uByte type;			// Socket type.mode
	DexSocket handle;	// socket handle

	//-------------------------------
	// Private SocketTable Variables
	//-------------------------------
	private:
	char buff[RECV_BUFF];
	uLong buff_used;
};


//-------------------------------------------------------------------
// DexterNET Class - Derived from NetEnvironment
//
// DexterNET contains all of Dexters Networking functions and Variables.
// The Class forms a 100% portable function interface.
// It inherits the functions and variables contained in NetEnvironment
// which is hidden from the user but forms the System Specific heart of
// the Class.
//-------------------------------------------------------------------
class DexterNET {
	public:
	DexterNET(void);	// Define Constructor
	~DexterNET(void);	// Define DeConstructor

	// Public DexterNET Variables
	static uLong err;	// Stores last error

	// Private DexterNET Variables
	static SocketTable Sock[DEX_MAX_SOCKETS];	// Dexters Socket Open tracker
	static bool NetActive;						// Availability of Networking

	// Public DexterNET Functions
	public:
	static bool Init(void);
	static bool PlatformInit(void);
	static void ShutDown(void);
	static bool Active(void);
	static DexSocket Connect(char* servname, uWord port);
	static void Disconnect(DexSocket);
	static bool IsConnected(DexSocket);
	static uLong Send(DexSocket, char* str, uLong ln);
	static uLong SendString(DexSocket, char* str);
	static sLong Read(DexSocket, char* str, uLong ln);
	static sLong ReadString(DexSocket, char* str, uLong ln);
	static void SetSocketBlocking(DexSocket socket, bool block);
	static DexSocket NetBind(uWord port);
	static DexSocket NetAcceptConnection(DexSocket OpenSocket);
	static void ErrorLog(const uLong error);

	// Private DexterNET Functions
	private:
	static sWord FindSocketId(DexSocket socket);
	static sWord FindFreeSocketId(void);
};


#endif	// _DEXTER_NET_H

